@@ -11,42 +11,76 @@ print.summary.meta <- function(x,
                                ...){
   
   
-  if (!inherits(x, "summary.meta"))
-    stop("Argument 'x' must be an object of class \"summary.meta\"")
-
-  if (inherits(x, "metacum"))
+  ##
+  ##
+  ## (1) Check for summary.meta object
+  ##
+  ##
+  chkclass(x, "summary.meta")
+  ##
+  if (inherits(x, "metacum") | inherits(x, "metainf"))
     return(invisible(NULL))
   ##
-  if (inherits(x, "metainf"))
-    return(invisible(NULL))
+  by <- !is.null(x$bylab)
   
   
+  ##
+  ##
+  ## (2) Check other arguments
+  ##
+  ##
+  chknumeric(digits)
+  chklogical(comb.fixed)
+  chklogical(comb.random)
+  chklogical(prediction)
+  if (by)
+    chklogical(print.byvar)
+  if (!is.null(print.CMH))
+    chklogical(print.CMH)
+  chklogical(header)
+  chklogical(backtransf)
+  chknumeric(bylab.nchar)
+  ##
+  ## Additional arguments / checks for metacont objects
+  ##
   cl <- class(x)[1]
   addargs <- names(list(...))
   ##
   fun <- "print.summary.meta"
   ##
   warnarg("logscale", addargs, fun, otherarg="backtransf")
+  
+  
+  ##
+  ##
+  ## (3) Some additional settings
+  ##
   ##
-  if (is.null(backtransf))
-    if (!is.null(list(...)[["logscale"]]))
-      backtransf <- !list(...)[["logscale"]]
-    else
-      backtransf <- TRUE
-  
-  
-  bip <- inherits(x, c("metabin", "metainc", "metaprop"))
-  
-  
   k <- x$k
   sm <- x$sm
   ##
+  bip <- inherits(x, c("metabin", "metainc", "metaprop"))
+  ##
+  prediction <- prediction & comb.random & k>=3
+  ##
   if (is.null(x$df.Q))
     df.Q <- k-1
   else
     df.Q <- x$df.Q
-  
-  
+  ##
+  if (by){
+    k.w <- x$k.w
+    if (is.null(x$df.Q.w))
+      df.Q.w <- sum((k.w-1)[!is.na(x$Q.w)])
+    else
+      df.Q.w <- x$df.Q.w
+    ##
+    if (is.null(x$df.Q.b))
+      df.Q.b <- (k-1) - sum((k.w-1)[!is.na(x$Q.w)])
+    else
+      df.Q.b <- x$df.Q.b
+  }
+  ##  
   sm.lab <- sm
   ##
   if (backtransf){
@@ -58,32 +92,24 @@ print.summary.meta <- function(x,
   else 
     if (is.relative.effect(sm))
       sm.lab <- paste("log", sm, sep="")
-  
-  
-  by <- !is.null(x$bylab)
-  
-  
-  if (length(comb.fixed)==0)
-    comb.fixed <- TRUE
-  ##
-  if (length(comb.random)==0)
-    comb.random <- TRUE
-  ##
-  if (length(prediction)==0)
-    prediction <- FALSE
-  ##
-  if (length(print.byvar)==0)
-    print.byvar <- TRUE
-  ##
-  if (length(print.CMH)==0)
-    print.CMH <- FALSE
   ##
   if (length(x$tau.common)==0)
     x$tau.common <- FALSE
+  ##
+  if (length(x$tau.common)==0)
+    x$tau.common <- FALSE
+  ##
+  if (by)
+    bylevs <- ifelse(nchar(x$bylevs) > bylab.nchar,
+                     paste(substring(x$bylevs, 1, bylab.nchar-4), " ...", sep=""),#
+                     x$bylevs)
   
   
-  prediction <- prediction & comb.random & k>=3
-  
+  ##
+  ##
+  ## (4) Set and backtransform results of meta-analysis
+  ##
+  ##
   TE.fixed    <- x$fixed$TE
   lowTE.fixed <- x$fixed$lower
   uppTE.fixed <- x$fixed$upper
@@ -100,17 +126,14 @@ print.summary.meta <- function(x,
     lowTE.fixed.w  <- x$within.fixed$lower
     uppTE.fixed.w  <- x$within.fixed$upper
     pval.fixed.w   <- x$within.fixed$p
-    harmonic.mean.fixed.w <- x$within.fixed$harmonic.mean
+    n.harmonic.mean.w <- x$within.fixed$harmonic.mean
     TE.random.w    <- x$within.random$TE
     lowTE.random.w <- x$within.random$lower
     uppTE.random.w <- x$within.random$upper
     pval.random.w   <- x$within.random$p
-    harmonic.mean.random.w <- x$within.random$harmonic.mean
   }
-  
-  
+  ##  
   if (backtransf){
-    ##
     npft.ma <- 1/mean(1/x$n)
     ##
     TE.fixed    <- backtransf(TE.fixed, sm, "mean",
@@ -133,7 +156,7 @@ print.summary.meta <- function(x,
                                 npft.ma, warn=prediction)
     ##
     if (by){
-      npft.w <- harmonic.mean.fixed.w
+      npft.w <- n.harmonic.mean.w
       ##
       TE.fixed.w     <- backtransf(TE.fixed.w, sm, "mean",
                                    npft.w, warn=comb.fixed)
@@ -150,8 +173,7 @@ print.summary.meta <- function(x,
                                    npft.w, warn=comb.random)
     }
   }
-  
-  
+  ##  
   TE.fixed    <- round(TE.fixed, digits)
   lowTE.fixed <- round(lowTE.fixed, digits)
   uppTE.fixed <- round(uppTE.fixed, digits)
@@ -167,8 +189,6 @@ print.summary.meta <- function(x,
   lowTE.predict <- round(lowTE.predict, digits)
   uppTE.predict <- round(uppTE.predict, digits)
   ##
-  k.w <- x$k.w
-  ##
   if (by){
     TE.fixed.w     <- round(TE.fixed.w, digits)
     lowTE.fixed.w  <- round(lowTE.fixed.w, digits)
@@ -187,28 +207,25 @@ print.summary.meta <- function(x,
   uppI2 <- x$I2$upper
   
   
-  if (by)
-    by.levs <- ifelse(nchar(x$by.levs) > bylab.nchar,
-                      paste(substring(x$by.levs, 1, bylab.nchar-4), " ...", sep=""),#
-                      x$by.levs)
-  
-  
+  ##
+  ##
+  ## (5) Print result for meta-analysis
+  ##
+  ##
   if (header)
     crtitle(x)
-  
-  
+  ##
   if (x$k.all == 1){
+    ##
+    ## Print results for a single study
+    ##
     res <- cbind(TE.fixed,
                  p.ci(format(lowTE.fixed), format(uppTE.fixed)),
                  format(round(zTE.fixed,4)),
                  format.p(pTE.fixed))
-    
-    dimnames(res) <- list("", c(sm.lab, x$ci.lab, "z", "p.value"))
-    
+    dimnames(res) <- list("", c(sm.lab, x$ci.lab, "z", "p-value"))
     prmatrix(res, quote=FALSE, right=TRUE, ...)
-    
     ## Print information on summary method:
-    ##
     catmeth(method=x$method,
             sm=sm,
             k.all=x$k.all,
@@ -224,14 +241,15 @@ print.summary.meta <- function(x,
             pooledvar=x$pooledvar)
   }
   else{
-
+    ##
+    ## Print results for meta-analysis with more than one study
+    ##
     if (comb.fixed|comb.random){
       if (!inherits(x, "trimfill"))
         cat(paste("Number of studies combined: k=", k, "\n\n", sep=""))
       else
         cat(paste("Number of studies combined: k=", k,
                   " (with ", x$k0, " added studies)\n\n", sep=""))
-      
       res <- cbind(format(c(if (comb.fixed) TE.fixed,
                             if (comb.random) TE.random,
                             if (prediction) NA)),
@@ -247,10 +265,8 @@ print.summary.meta <- function(x,
                    format.p(c(if (comb.fixed) pTE.fixed,
                               if (comb.random) pTE.random,
                               if (prediction) NA)))
-      
       if (prediction)
         res[dim(res)[1], c(1,3:4)] <- ""
-      
       if (!is.null(x$hakn) && x$hakn){
         if (comb.fixed & comb.random)
           zlab <- "z|t"
@@ -261,21 +277,17 @@ print.summary.meta <- function(x,
       }
       else
         zlab <- "z"
-      
-      
+      ##
       dimnames(res) <- list(c(if (comb.fixed) "Fixed effect model",
                               if (comb.random) "Random effects model",
                               if (prediction) "Prediction interval"),  
-                            c(sm.lab, x$ci.lab, zlab, "p.value"))
-      
+                            c(sm.lab, x$ci.lab, zlab, "p-value"))
       prmatrix(res, quote=FALSE, right=TRUE, ...)
-      
-      
-      if (inherits(x, "metabin") & print.CMH){
+      ##
+      if (inherits(x, "metabin") && print.CMH){
         Qdata <- cbind(round(x$Q.CMH, 2), 1,
                        format.p(1-pchisq(x$Q.CMH, df=1)))
-        
-        dimnames(Qdata) <- list("", c("Q", "d.f.", "p.value"))
+        dimnames(Qdata) <- list("", c("Q", "d.f.", "p-value"))
         ##
         cat("\nCochran-Mantel-Haenszel (CMH) test for overall effect: \n")
         prmatrix(Qdata, quote=FALSE, right=TRUE, ...)
@@ -283,19 +295,19 @@ print.summary.meta <- function(x,
     }
     else
       cat(paste("Number of studies: k=", k, "\n", sep=""))
-    
-
+    ##
+    ## Print information on heterogeneity
+    ##
     if (!is.na(x$tau))
       cat(paste("\nQuantifying heterogeneity:\n",
                 if (x$tau^2 > 0 & x$tau^2 < 0.0001)
-                "tau^2 < 0.0001"
+                paste("tau^2", format.tau(x$tau^2))
                 else
                 paste("tau^2 = ",
                       ifelse(x$tau==0,
                              "0",
                              format(round(x$tau^2, 4), 4, nsmall=4, scientific=FALSE)),
-                      sep="")
-                ,
+                      sep=""),
                 paste("; H = ", round(H, 2),
                       ifelse(k>2,
                              paste(" ", p.ci(round(lowH, 2), round(uppH, 2)), sep=""),
@@ -310,229 +322,119 @@ print.summary.meta <- function(x,
                              ""),
                       sep=""),
                 "\n", sep=""))
-    
-    
+    ##    
     if (k > 1 & (comb.fixed|comb.random)){
-      
       Qdata <- cbind(round(x$Q, 2), df.Q,
                      format.p(1-pchisq(x$Q, df=df.Q)))
-      
-      dimnames(Qdata) <- list("", c("Q", "d.f.", "p.value"))
+      dimnames(Qdata) <- list("", c("Q", "d.f.", "p-value"))
       ##
       cat("\nTest of heterogeneity:\n")
       prmatrix(Qdata, quote=FALSE, right=TRUE, ...)
       ##
       if (by){
-        if (comb.fixed==TRUE){
-          if (is.null(x$version) || as.numeric(strsplit(x$version, "-")[[1]][1]) < 1.7){
-            ##
-            ## R-version < 1.7-0:
-            ##
-            Q <- x$Q
-            Q.w <- sum(x$Q.w, na.rm=TRUE)
-            if (!is.null(x$Q.b.fixed))
-              Q.b <- x$Q.b.fixed
-            else
-              Q.b <- Q - Q.w
-            ##
-            if (x$method=="MH"){
-              Q.w <- NA
-              Q.b <- NA
-            }
-            ##
-            Qs  <- c(Q.b,  Q.w, x$Q.w)
-            Qs <- ifelse(Qs > -0.1 & Qs < 0, 0, Qs)
-
-            df <- k-1
-            df.w <- sum((x$k.w-1)[!is.na(x$Q.w)])
-            df.b <- df - df.w
-            ##
-            if (x$method=="MH"){
-              df.w <- NA
-              df.b <- NA
-            }
-            ##
-            dfs <- c(df.b, df.w, k.w-1)
-            dfs[dfs<=0] <- NA
-
-            pval <- 1-pchisq(Qs[1:2], df=dfs[1:2])
-            
-            Qdata <- cbind(ifelse(is.na(Qs),
-                                  "--",
-                                  format(round(Qs, 2))),
-                           ifelse(is.na(dfs), 0, dfs),
-                           c("--", "--", format(TE.fixed.w)),
-                           c("--", "--",
-                             p.ci(format(lowTE.fixed.w),
-                                  format(uppTE.fixed.w))),
-                           c(format.p(pval),
-                             rep("--", length(x$Q.w))))
-            
-            if (print.byvar)
-              bylab <- paste(x$bylab,
-                             " = ", 
-                             format(by.levs), sep="")
-            else
-              bylab <- format(by.levs)
-            
-            
-            dimnames(Qdata) <- list(c("Between groups  ",
-                                      "Within groups   ", 
-                                      bylab),
-                                    c("Q", "d.f.", sm.lab, x$ci.lab,
-                                      "p.value"))
-            
-            
-            if (x$method=="MH"){
-              warning("Test for subgroup differences for Mantel-Haenszel method only available in newer version (> 1.6-1) of R package meta")
-              Qdata <- Qdata[-(2:3),]
-            }
-            ##
-            cat("\nTest for subgroup differences (fixed effect model):\n")
+        ##
+        ## Print information for subgroup analysis
+        ##
+        if (comb.fixed){
+          ##
+          ## Subgroup analysis based on fixed effect model
+          ##
+          Q.w <- ifelse(is.na(x$Q.w),
+                        "--",
+                        format(round(x$Q.w, 2)))
+          I2.w <- ifelse(is.na(x$I2.w$TE),
+                         "--",
+                         paste(round(100*x$I2.w$TE, 1), "%", sep=""))
+          tau2.w <- ifelse(k.w==1, "--", format.tau(x$tau.w^2))
+          ##
+          Tdata <- cbind(format(k.w),
+                         format(TE.fixed.w),
+                         c(p.ci(format(lowTE.fixed.w), format(uppTE.fixed.w))),
+                         Q.w, tau2.w, I2.w
+                         ) #, format.p(pval.fixed.w))
+          if (print.byvar)
+            bylab <- paste(x$bylab,
+                           " = ", 
+                           format(bylevs), sep="")
+          else
+            bylab <- format(bylevs)
+          dimnames(Tdata) <- list(bylab,
+                                  c("  k", sm.lab, x$ci.lab,
+                                    "Q", "tau^2", "I^2")
+                                  ) #, "p-value"))
+          cat("\nResults for subgroups (fixed effect model):\n")
+          prmatrix(Tdata, quote=FALSE, right=TRUE, ...)
+          ##
+          cat("\nTest for subgroup differences (fixed effect model):\n")
+          if (x$method=="MH"){
+            Qdata <- cbind(round(x$Q.b.fixed, 2), df.Q.b,
+                           format.p(1-pchisq(x$Q.b.fixed, df=df.Q.b)))
+            dimnames(Qdata) <- list("Between groups  ",
+                                    c("Q", "d.f.", "p-value"))
             prmatrix(Qdata, quote=FALSE, right=TRUE, ...)
           }
           else{
-            ##
-            ## R-version >= 1.7-0:
-            ##
-            Q.w <- ifelse(is.na(x$Q.w),
-                          "--",
-                          format(round(x$Q.w, 2)))
-            I2.w <- ifelse(is.na(x$I2.w$TE),
-                           "--",
-                           paste(round(100*x$I2.w$TE, 1), "%", sep=""))
-            tau2.w <- ifelse(x$k.w==1, "--", format.tau(x$tau.w^2))
-            ##
-            Tdata <- cbind(format(x$k.w),
-                           format(TE.fixed.w),
-                           c(p.ci(format(lowTE.fixed.w), format(uppTE.fixed.w))),
-                           Q.w, tau2.w, I2.w
-                           ) #, format.p(pval.fixed.w))
-            if (print.byvar)
-              bylab <- paste(x$bylab,
-                             " = ", 
-                             format(by.levs), sep="")
-            else
-              bylab <- format(by.levs)
-            dimnames(Tdata) <- list(bylab,
-                                    c("  k", sm.lab, x$ci.lab,
-                                      "Q", "tau^2", "I^2")
-                                    ) #, "p.value"))
-            cat("\nResults for subgroups (fixed effect model):\n")
-            prmatrix(Tdata, quote=FALSE, right=TRUE, ...)
-            
-            cat("\nTest for subgroup differences (fixed effect model):\n")
-            df.b <- (k-1) - sum((x$k.w-1)[!is.na(x$Q.w)])
-            ##
-            if (x$method=="MH"){
-              Qdata <- cbind(round(x$Q.b.fixed, 2), df.b,
-                             format.p(1-pchisq(x$Q.b.fixed, df=df.b)))
-              dimnames(Qdata) <- list("Between groups  ",
-                                      c("Q", "d.f.", "p.value"))
-              prmatrix(Qdata, quote=FALSE, right=TRUE, ...)
-            }
-            else{
-              Qs  <- round(c(x$Q.b.fixed, sum(x$Q.w, na.rm=TRUE)), 2)
-              dfs <- c(df.b, sum(k.w-1))
-              Qdata <- cbind(Qs,
-                             dfs,
-                             format.p(1-pchisq(Qs, df=dfs)))
-              dimnames(Qdata) <- list(c("Between groups", "Within groups"),
-                                      c("Q", "d.f.", "p.value"))
-              prmatrix(Qdata, quote=FALSE, right=TRUE, ...)
-            }
+            Qs  <- c(x$Q.b.fixed, x$Q.w.fixed)
+            dfs <- c(df.Q.b, df.Q.w)
+            Qdata <- cbind(round(Qs, 2),
+                           dfs,
+                           format.p(1-pchisq(Qs, df=dfs)))
+            dimnames(Qdata) <- list(c("Between groups", "Within groups"),
+                                    c("Q", "d.f.", "p-value"))
+            prmatrix(Qdata, quote=FALSE, right=TRUE, ...)
           }
         }
         ##
-        ##
-        if (comb.random==TRUE){
-          if (is.null(x$version) || as.numeric(strsplit(x$version, "-")[[1]][1]) < 1.7){
-            ##
-            ## R-version < 1.7-0:
-            ##
-            Q <- x$Q
-            if (!is.null(x$Q.b.random))
-              Q.b <- x$Q.b.random
-            else
-              Q.b <- NA
-            Qs <- c(Q.b, rep(NA, length(x$k.w)))
-            Qs <- ifelse(Qs > -0.1 & Qs < 0, 0, Qs)
-            
-            df <- k-1
-            df.w <- sum((x$k.w-1)[!is.na(x$Q.w)])
-            df.b <- df - df.w
-            dfs <- c(df.b, k.w-1)
-            dfs[dfs<=0] <- NA
-            
-            pval <- 1-pchisq(Qs[1], df=dfs[1])
-            
-            fQs <- rmSpace(as.character(format(round(Qs, 2))))
-            
-            Qdata <- cbind(ifelse(fQs=="NA", "--", fQs),
-                           ifelse(is.na(dfs), 0, dfs),
-                           c("--", format(TE.random.w)),
-                           c("--", p.ci(format(lowTE.random.w),
-                                        format(uppTE.random.w))),
-                           c(format.p(pval), rep("--", length(x$k.w))))
-            
-            if (print.byvar)
-              bylab <- paste(x$bylab,
-                             " = ", 
-                             format(by.levs), sep="")
-            else
-              bylab <- format(by.levs)
-            
-            
-            dimnames(Qdata) <- list(c("Between groups  ", bylab),
-                                    c("Q", "d.f.", sm.lab, x$ci.lab,
-                                      "p.value"))
-            ##
-            cat("\nTest for subgroup differences (random effects model):\n")
-            prmatrix(Qdata, quote=FALSE, right=TRUE, ...)
+        if (comb.random){
+          ##
+          ## Subgroup analysis based on random effects model
+          ##
+          Q.w <- ifelse(is.na(x$Q.w),
+                        "--",
+                        format(round(x$Q.w, 2)))
+          I2.w <- ifelse(is.na(x$I2.w$TE),
+                         "--",
+                         paste(round(100*x$I2.w$TE, 1), "%", sep=""))
+          tau2.w <- ifelse(k.w==1, "--", format.tau(x$tau.w^2))
+          ##
+          Tdata <- cbind(format(k.w),
+                         format(TE.random.w),
+                         c(p.ci(format(lowTE.random.w), format(uppTE.random.w))),
+                         Q.w, tau2.w, I2.w
+                         )
+          if (print.byvar)
+            bylab <- paste(x$bylab,
+                           " = ", 
+                           format(bylevs), sep="")
+          else
+            bylab <- format(bylevs)
+          dimnames(Tdata) <- list(bylab,
+                                  c("  k", sm.lab, x$ci.lab,
+                                    "Q", "tau^2", "I^2")
+                                  ) #, "p-value"))
+          cat("\nResults for subgroups (random effects model):\n")
+          prmatrix(Tdata, quote=FALSE, right=TRUE, ...)
+          ##
+          cat("\nTest for subgroup differences (random effects model):\n")
+          if (is.na(x$Q.w.random)){
+            Qdata <- cbind(round(x$Q.b.random, 2), df.Q.b,
+                           format.p(1-pchisq(x$Q.b.random, df=df.Q.b)))
+          dimnames(Qdata) <- list("Between groups  ",
+                                  c("Q", "d.f.", "p-value"))
           }
           else{
-            ##
-            ## R-version >= 1.7-0:
-            ##
-            Q.w <- ifelse(is.na(x$Q.w),
-                          "--",
-                          format(round(x$Q.w, 2)))
-            I2.w <- ifelse(is.na(x$I2.w$TE),
-                           "--",
-                           paste(round(100*x$I2.w$TE, 1), "%", sep=""))
-            tau2.w <- ifelse(x$k.w==1, "--", format.tau(x$tau.w^2))
-            ##
-            Tdata <- cbind(format(x$k.w),
-                           format(TE.random.w),
-                           c(p.ci(format(lowTE.random.w), format(uppTE.random.w))),
-                           Q.w, tau2.w, I2.w
-                           ) #, format.p(pval.random.w))
-            if (print.byvar)
-              bylab <- paste(x$bylab,
-                             " = ", 
-                             format(by.levs), sep="")
-            else
-              bylab <- format(by.levs)
-            dimnames(Tdata) <- list(bylab,
-                                    c("  k", sm.lab, x$ci.lab,
-                                      "Q", "tau^2", "I^2")
-                                    ) #, "p.value"))
-            cat("\nResults for subgroups (random effects model):\n")
-            prmatrix(Tdata, quote=FALSE, right=TRUE, ...)
-            
-            cat("\nTest for subgroup differences (random effects model):\n")
-            df.b <- (k-1) - sum((x$k.w-1)[!is.na(x$Q.w)])
-            ##
-            Qdata <- cbind(round(x$Q.b.random, 2), df.b,
-                           format.p(1-pchisq(x$Q.b.random, df=df.b)))
-            dimnames(Qdata) <- list("Between groups  ",
-                                    c("Q", "d.f.", "p.value"))
-            prmatrix(Qdata, quote=FALSE, right=TRUE, ...)
+            Qs  <- c(x$Q.b.random, x$Q.w.random)
+            dfs <- c(df.Q.b, df.Q.w)
+            Qdata <- cbind(round(Qs, 2),
+                           dfs, format.p(1-pchisq(Qs, df=dfs)))
+            dimnames(Qdata) <- list(c("Between groups", "Within groups"),
+                                    c("Q", "d.f.", "p-value"))
           }
+          prmatrix(Qdata, quote=FALSE, right=TRUE, ...)
         }
       }
     }
-
+    ##
     ## Print information on summary method:
     ##
     catmeth(method=x$method,
@@ -556,5 +458,6 @@ print.summary.meta <- function(x,
             pooledvar=x$pooledvar)
   }
   
+  
   invisible(NULL)
 }
